@@ -14,7 +14,6 @@
 #    under the License.
 from __future__ import print_function
 
-import json
 import os
 import prettytable
 import sys
@@ -24,9 +23,10 @@ import yaml
 
 from heatclient import exc
 from heatclient.openstack.common import importutils
+from heatclient.openstack.common import jsonutils
 
 supported_formats = {
-    "json": lambda x: json.dumps(x, indent=2),
+    "json": lambda x: jsonutils.dumps(x, indent=2),
     "yaml": yaml.safe_dump
 }
 
@@ -34,7 +34,7 @@ supported_formats = {
 # Decorator for cli-args
 def arg(*args, **kwargs):
     def _decorator(func):
-        # Because of the sematics of decorator composition if we just append
+        # Because of the semantics of decorator composition if we just append
         # to the options list positional options will appear to be backwards.
         func.__dict__.setdefault('arguments', []).insert(0, (args, kwargs))
         return func
@@ -46,7 +46,7 @@ def link_formatter(links):
 
 
 def json_formatter(js):
-    return json.dumps(js, indent=2)
+    return jsonutils.dumps(js, indent=2)
 
 
 def text_wrap_formatter(d):
@@ -116,10 +116,6 @@ def find_resource(manager, name_or_id):
         raise exc.CommandError(msg)
 
 
-def string_to_bool(arg):
-    return arg.strip().lower() in ('t', 'true', 'yes', '1')
-
-
 def env(*vars, **kwargs):
     """Search for the first defined of possibly many env vars
 
@@ -166,7 +162,13 @@ def format_parameters(params):
                                    'Use the key=value format')
             raise exc.CommandError(msg)
 
-        parameters[n] = v
+        if n not in parameters:
+            parameters[n] = v
+        else:
+            if not isinstance(parameters[n], list):
+                parameters[n] = [parameters[n]]
+            parameters[n].append(v)
+
     return parameters
 
 
