@@ -13,101 +13,18 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import json
-import os
-import urllib
 import yaml
 
+from heatclient.common import template_utils
 from heatclient.common import utils
-from heatclient.openstack.common.py3kcompat import urlutils
+from heatclient.openstack.common import jsonutils
 
 import heatclient.exc as exc
 
 
-def _set_template_fields(hc, args, fields):
-    if args.template_file:
-        tpl = open(args.template_file).read()
-        if tpl.startswith('{'):
-            fields['template'] = json.loads(tpl)
-        else:
-            fields['template'] = tpl
-    elif args.template_url:
-        fields['template_url'] = args.template_url
-    elif args.template_object:
-        template_body = hc.raw_request('GET', args.template_object)
-        if template_body:
-            fields['template'] = json.loads(template_body)
-        else:
-            raise exc.CommandError('Could not fetch template from %s'
-                                   % args.template_object)
-    else:
-        raise exc.CommandError('Need to specify exactly one of '
-                               '--template-file, --template-url '
-                               'or --template-object')
-
-
-def _get_file_contents(resource_registry, fields, base_url='',
-                       ignore_if=None):
-    for key, value in iter(resource_registry.items()):
-        if ignore_if and ignore_if(key, value):
-            continue
-
-        if base_url != '' and not base_url.endswith('/'):
-            base_url = base_url + '/'
-        str_url = base_url + value
-        try:
-            fields['files'][str_url] = urlutils.urlopen(str_url).read()
-        except urlutils.URLError:
-            raise exc.CommandError('Could not fetch %s from the environment'
-                                   % str_url)
-        resource_registry[key] = str_url
-
-
-def _prepare_environment_file(environment_file):
-    environment_dir = os.path.dirname(os.path.abspath(environment_file))
-    environment_url = urlutils.urljoin(
-        'file:', urllib.pathname2url(environment_dir))
-
-    raw_env = open(environment_file).read()
-    env = yaml.safe_load(raw_env)
-    return environment_url, env
-
-
-def _process_environment_and_files(args, fields):
-    if not args.environment_file:
-        return
-
-    environment_url, env = _prepare_environment_file(args.environment_file)
-    _resolve_environment_urls(fields, environment_url, env)
-
-
-def _resolve_environment_urls(fields, environment_url, env):
-    fields['environment'] = env
-    fields['files'] = {}
-
-    rr = env.get('resource_registry', {})
-    base_url = rr.get('base_url', environment_url)
-
-    def ignore_if(key, value):
-        if key == 'base_url':
-            return True
-        if isinstance(value, dict):
-            return True
-        if '::' in value:
-            # Built in providers like: "X::Compute::Server"
-            # don't need downloading.
-            return True
-
-    _get_file_contents(rr, fields, base_url, ignore_if)
-
-    for res_name, res_dict in iter(rr.get('resources', {}).items()):
-        res_base_url = res_dict.get('base_url', base_url)
-        _get_file_contents(res_dict, fields, res_base_url, ignore_if)
-
-
 @utils.arg('-f', '--template-file', metavar='<FILE>',
            help='Path to the template.')
-@utils.arg('-e', '--environment-file', metavar='<FILE>',
+@utils.arg('-e', '--environment-file', metavar='<FILE or URL>',
            help='Path to the environment.')
 @utils.arg('-u', '--template-url', metavar='<URL>',
            help='URL of template.')
@@ -132,7 +49,7 @@ def do_create(hc, args):
 
 @utils.arg('-f', '--template-file', metavar='<FILE>',
            help='Path to the template.')
-@utils.arg('-e', '--environment-file', metavar='<FILE>',
+@utils.arg('-e', '--environment-file', metavar='<FILE or URL>',
            help='Path to the environment.')
 @utils.arg('-u', '--template-url', metavar='<URL>',
            help='URL of template.')
@@ -152,33 +69,65 @@ def do_create(hc, args):
            help='Name of the stack to create.')
 def do_stack_create(hc, args):
     '''Create the stack.'''
-    fields = {'stack_name': args.name,
-              'timeout_mins': args.create_timeout,
-              'disable_rollback': not(args.enable_rollback),
-              'parameters': utils.format_parameters(args.parameters)}
-    _set_template_fields(hc, args, fields)
-    _process_environment_and_files(args, fields)
+    tpl_files, template = template_utils.get_template_contents(
+        args.template_file,
+        args.template_url,
+        args.template_object,
+        hc.http_client.raw_request)
+    env_files, env = template_utils.process_environment_and_files(
+        env_path=args.environment_file)
+
+    fields = {
+        'stack_name': args.name,
+        'timeout_mins': args.create_timeout,
+        'disable_rollback': not(args.enable_rollback),
+        'parameters': utils.format_parameters(args.parameters),
+        'template': template,
+        'files': dict(tpl_files.items() + env_files.items()),
+        'environment': env
+    }
 
     hc.stacks.create(**fields)
     do_stack_list(hc)
 
 
-@utils.arg('id', metavar='<NAME or ID>', help='Name or ID of stack to delete.')
+@utils.arg('id', metavar='<NAME or ID>', nargs='+',
+           help='Name or ID of stack(s) to delete.')
 def do_delete(hc, args):
     '''DEPRECATED! Use stack-delete instead.'''
     do_stack_delete(hc, args)
 
 
-@utils.arg('id', metavar='<NAME or ID>', help='Name or ID of stack to delete.')
+@utils.arg('id', metavar='<NAME or ID>', nargs='+',
+           help='Name or ID of stack(s) to delete.')
 def do_stack_delete(hc, args):
-    '''Delete the stack.'''
+    '''Delete the stack(s).'''
+    failure_count = 0
+
+    for sid in args.id:
+        fields = {'stack_id': sid}
+        try:
+            hc.stacks.delete(**fields)
+        except exc.HTTPNotFound as e:
+            failure_count += 1
+            print(e)
+    if failure_count == len(args.id):
+        raise exc.CommandError("Unable to delete any of the specified "
+                               "stacks.")
+    do_stack_list(hc)
+
+
+@utils.arg('id', metavar='<NAME or ID>',
+           help='Name or ID of stack to abandon.')
+def do_stack_abandon(hc, args):
+    '''Abandon the stack.'''
     fields = {'stack_id': args.id}
     try:
-        hc.stacks.delete(**fields)
+        stack = hc.stacks.abandon(**fields)
     except exc.HTTPNotFound:
         raise exc.CommandError('Stack not found: %s' % args.id)
     else:
-        do_stack_list(hc)
+        print(jsonutils.dumps(stack, indent=2))
 
 
 @utils.arg('id', metavar='<NAME or ID>',
@@ -236,7 +185,7 @@ def do_stack_show(hc, args):
 
 @utils.arg('-f', '--template-file', metavar='<FILE>',
            help='Path to the template.')
-@utils.arg('-e', '--environment-file', metavar='<FILE>',
+@utils.arg('-e', '--environment-file', metavar='<FILE or URL>',
            help='Path to the environment.')
 @utils.arg('-u', '--template-url', metavar='<URL>',
            help='URL of template.')
@@ -256,7 +205,7 @@ def do_update(hc, args):
 
 @utils.arg('-f', '--template-file', metavar='<FILE>',
            help='Path to the template.')
-@utils.arg('-e', '--environment-file', metavar='<FILE>',
+@utils.arg('-e', '--environment-file', metavar='<FILE or URL>',
            help='Path to the environment.')
 @utils.arg('-u', '--template-url', metavar='<URL>',
            help='URL of template.')
@@ -271,28 +220,75 @@ def do_update(hc, args):
            help='Name or ID of stack to update.')
 def do_stack_update(hc, args):
     '''Update the stack.'''
-    fields = {'stack_id': args.id,
-              'parameters': utils.format_parameters(args.parameters)}
-    _set_template_fields(hc, args, fields)
-    _process_environment_and_files(args, fields)
+
+    tpl_files, template = template_utils.get_template_contents(
+        args.template_file,
+        args.template_url,
+        args.template_object,
+        hc.http_client.raw_request)
+
+    env_files, env = template_utils.process_environment_and_files(
+        env_path=args.environment_file)
+
+    fields = {
+        'stack_id': args.id,
+        'parameters': utils.format_parameters(args.parameters),
+        'template': template,
+        'files': dict(tpl_files.items() + env_files.items()),
+        'environment': env
+    }
 
     hc.stacks.update(**fields)
-    do_list(hc)
+    do_stack_list(hc)
 
 
-def do_list(hc, args={}):
+def do_list(hc, args=None):
     '''DEPRECATED! Use stack-list instead.'''
-    do_stack_list(hc, args)
+    do_stack_list(hc)
 
 
-def do_stack_list(hc, args={}):
+@utils.arg('-f', '--filters', metavar='<KEY1=VALUE1;KEY2=VALUE2...>',
+           help='Filter parameters to apply on returned stacks. '
+           'This can be specified multiple times, or once with parameters '
+           'separated by semicolon.',
+           action='append')
+@utils.arg('-l', '--limit', metavar='<LIMIT>',
+           help='Limit the number of stacks returned')
+@utils.arg('-m', '--marker', metavar='<ID>',
+           help='Only return stacks that appear after the given stack ID')
+def do_stack_list(hc, args=None):
     '''List the user's stacks.'''
     kwargs = {}
+    if args:
+        kwargs = {'limit': args.limit,
+                  'marker': args.marker,
+                  'filters': utils.format_parameters(args.filters)}
+
     stacks = hc.stacks.list(**kwargs)
     fields = ['id', 'stack_name', 'stack_status', 'creation_time']
     utils.print_list(stacks, fields, sortby=3)
 
 
+def do_resource_type_list(hc, args={}):
+    '''List the available resource types.'''
+    kwargs = {}
+    types = hc.resource_types.list(**kwargs)
+    utils.print_list(types, ['resource_type'], sortby=0)
+
+
+@utils.arg('resource_type', metavar='<RESOURCE_TYPE>',
+           help='Resource Type to get the details for.')
+def do_resource_type_show(hc, args={}):
+    '''Show the resource type.'''
+    try:
+        resource_type = hc.resource_types.get(args.resource_type)
+    except exc.HTTPNotFound:
+        raise exc.CommandError(
+            'Resource Type not found: %s' % args.resource_type)
+    else:
+        print(jsonutils.dumps(resource_type, indent=2))
+
+
 @utils.arg('id', metavar='<NAME or ID>',
            help='Name or ID of stack to get the template for.')
 def do_gettemplate(hc, args):
@@ -311,16 +307,16 @@ def do_template_show(hc, args):
         raise exc.CommandError('Stack not found: %s' % args.id)
     else:
         if 'heat_template_version' in template:
-            print yaml.safe_dump(template, indent=2)
+            print(yaml.safe_dump(template, indent=2))
         else:
-            print json.dumps(template, indent=2)
+            print(jsonutils.dumps(template, indent=2))
 
 
 @utils.arg('-u', '--template-url', metavar='<URL>',
            help='URL of template.')
 @utils.arg('-f', '--template-file', metavar='<FILE>',
            help='Path to the template.')
-@utils.arg('-e', '--environment-file', metavar='<FILE>',
+@utils.arg('-e', '--environment-file', metavar='<FILE or URL>',
            help='Path to the environment.')
 @utils.arg('-o', '--template-object', metavar='<URL>',
            help='URL to retrieve template object (e.g from swift)')
@@ -338,7 +334,7 @@ def do_validate(hc, args):
            help='URL of template.')
 @utils.arg('-f', '--template-file', metavar='<FILE>',
            help='Path to the template.')
-@utils.arg('-e', '--environment-file', metavar='<FILE>',
+@utils.arg('-e', '--environment-file', metavar='<FILE or URL>',
            help='Path to the environment.')
 @utils.arg('-o', '--template-object', metavar='<URL>',
            help='URL to retrieve template object (e.g from swift)')
@@ -349,12 +345,24 @@ def do_validate(hc, args):
            action='append')
 def do_template_validate(hc, args):
     '''Validate a template with parameters.'''
-    fields = {'parameters': utils.format_parameters(args.parameters)}
-    _set_template_fields(hc, args, fields)
-    _process_environment_and_files(args, fields)
+
+    tpl_files, template = template_utils.get_template_contents(
+        args.template_file,
+        args.template_url,
+        args.template_object,
+        hc.http_client.raw_request)
+
+    env_files, env = template_utils.process_environment_and_files(
+        env_path=args.environment_file)
+    fields = {
+        'parameters': utils.format_parameters(args.parameters),
+        'template': template,
+        'files': dict(tpl_files.items() + env_files.items()),
+        'environment': env
+    }
 
     validation = hc.stacks.validate(**fields)
-    print json.dumps(validation, indent=2)
+    print(jsonutils.dumps(validation, indent=2))
 
 
 @utils.arg('id', metavar='<NAME or ID>',
@@ -410,7 +418,8 @@ def do_resource_show(hc, args):
 @utils.arg('resource', metavar='<RESOURCE>',
            help='Name of the resource to generate a template for.')
 @utils.arg('-F', '--format', metavar='<FORMAT>',
-           help="The template output format. %s" % utils.supported_formats)
+           help="The template output format, one of: %s"
+                % ', '.join(utils.supported_formats.keys()))
 def do_resource_template(hc, args):
     '''Generate a template based on a resource.'''
     fields = {'resource_name': args.resource}
@@ -420,9 +429,9 @@ def do_resource_template(hc, args):
         raise exc.CommandError('Resource %s not found.' % args.resource)
     else:
         if args.format:
-            print utils.format_output(template, format=args.format)
+            print(utils.format_output(template, format=args.format))
         else:
-            print utils.format_output(template)
+            print(utils.format_output(template))
 
 
 @utils.arg('id', metavar='<NAME or ID>',
@@ -439,7 +448,7 @@ def do_resource_metadata(hc, args):
         raise exc.CommandError('Stack or resource not found: %s %s' %
                                (args.id, args.resource))
     else:
-        print json.dumps(metadata, indent=2)
+        print(jsonutils.dumps(metadata, indent=2))
 
 
 @utils.arg('id', metavar='<NAME or ID>',
@@ -452,8 +461,10 @@ def do_event_list(hc, args):
               'resource_name': args.resource}
     try:
         events = hc.events.list(**fields)
-    except exc.HTTPNotFound:
-        raise exc.CommandError('Stack not found: %s' % args.id)
+    except exc.HTTPNotFound as ex:
+        # it could be the stack or resource that is not found
+        # just use the message that the server sent us.
+        raise exc.CommandError(str(ex))
     else:
         fields = ['id', 'resource_status_reason',
                   'resource_status', 'event_time']
@@ -499,3 +510,13 @@ def do_event_show(hc, args):
             'resource_properties': utils.json_formatter
         }
         utils.print_dict(event.to_dict(), formatters=formatters)
+
+
+def do_build_info(hc, args):
+    '''Retrieve build information.'''
+    result = hc.build_info.build_info()
+    formatters = {
+        'api': utils.json_formatter,
+        'engine': utils.json_formatter,
+    }
+    utils.print_dict(result, formatters=formatters)
